/*
 * QtElementList.h
 *
 *  Created on: Mar 6, 2012
 *      Author: ankele
 */

#ifndef QTELEMENTLIST_H_
#define QTELEMENTLIST_H_

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <QtGui/QTreeWidget>

#include <boost/thread/recursive_mutex.hpp>

#include "types.hpp"

#include "UIElements/Qt4/InstanceBoard/QtObservedAtom.hpp"
#include "UIElements/Views/Qt4/ElementList/QtObservedAtomObserver.hpp"

class QtObservedInstanceBoard;

class QtElementList: public QTreeWidget
{
  Q_OBJECT

public:
  QtElementList(QtObservedInstanceBoard *_board, QWidget * _parent=0);
  virtual ~QtElementList();

protected:
  virtual void paintEvent(QPaintEvent * event);

  void setOccurrence(QTreeWidgetItem &_item, const int count);

  static const int COLUMNCOUNT;
  enum COLUMNTYPES {NUMBER,NAME,SYMBOL,MASS,OCCURRENCE,COLUMNTYPES_MAX};
  static const char *COLUMNNAMES[];

private slots:
  void rowSelected();
  void changeElement(const QtObservedAtom::ptr _atom);
  void addAtomElement(const QtObservedAtom::ptr _atom);
  void removeAtomElement(const atomId_t _atom);

private:
  std::vector<bool> elementSelection; //!< needed to determine when a selection changes
  typedef std::map<const atomId_t, atomicNumber_t> AtomElementMap_t;
  AtomElementMap_t AtomElementMap;
  bool dirty;

  //!> observed all QtObservedAtoms for ElementChanged signals
  QtObservedAtomObserver atomobserver;

  mutable boost::recursive_mutex refill_mutex;
};

#endif /* QTELEMENTLIST_H_ */
